/*
  (C)2019 izumi@ieee.org, all rights reserved
*/

#ifndef GEISTERLIB_H

#define GEISTERLIB_H 1

#include <stdio.h>

// 6x6 board
// north-west corner is (0,0)
// Your ghosts are on the south side and faced to the north.

// Players
#define GstPly0  0  // your ghost
#define GstPly1  1  // opponent's ghost
#define GstNone -1  // none or empty
#define GstOppositePlayer(X) (1-(X))

// Ghost Colors
#define GstRed      0
#define GstBlue     1
#define GstUnknown -1

// Ghost States
#define GstOnboard   0
#define GstCaptured  1
#define GstEscaped  -1

// Board Width and Height
#define GstBdW 6
#define GstBdH 6

// Directions
#define GstNorth 0
#define GstEast  1
#define GstSouth 2
#define GstWest  3

// Game States
#define GstWin       1
#define GstLose     -1
#define GstContinue  0

struct GstGhostStr {
  char player;
  char id;
  char color;
  char state;
  char x, y;
};

struct GstGameStr {
  struct GstGhostStr ghost[2][8]; // 2players x 8ghosts
  struct GstGhostStr *board[GstBdW][GstBdH];
  char capturecount[2][2]; // 2players x {red,blue}
  int turn;
};

struct GstActionStr {
  struct GstGhostStr *ghost;
  char dir;
};

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifdef GEISTERLIB_C
const char dir2dx[4]={0,+1,0,-1};
const char dir2dy[4]={-1,0,+1,0};
const char oppositedir[4]={GstSouth,GstWest,GstNorth,GstEast};
FILE *gstlog=NULL;
#else
extern char dir2dx[];
extern char dir2dy[];
extern char oppositedir[];
extern FILE *gstlog;
#endif

int GstLogOpen(void);
int GstLogClose(void);
int GstInitGame(struct GstGameStr *s);
int GstCopyGame(struct GstGameStr *dst, struct GstGameStr *src, int reverse);
int GstPrintGhostChar(FILE *fp, struct GstGhostStr *g);
int GstPrintBoard(FILE *fp, struct GstGameStr *s);
int GstPrintGame(FILE *fp, struct GstGameStr *s);
int GstApplyAction(struct GstGameStr *s, struct GstActionStr *a, int exec);
#define GstCheckAction(S,A) GstApplyAction(S,A,0)
#define GstExecAction(S,A) GstApplyAction(S,A,1)
int GstJudgeGame(struct GstGameStr *s);
int GstCheckGame(struct GstGameStr *s);

int WeightedRandom(float w[], int n);

void GstDie(char *msg); // defined in the file including main()

#endif
